﻿using ProjectMonorail.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectMonorail.Drawnings;

public class DrawningLocomotive
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityLocomotive? EntityLocomotive { get; protected set; }
    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;
    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;
    /// <summary>
    /// Левая координата прорисовки автомобиля
    /// </summary>
    protected int? _startPosX;
    /// <summary>
    /// Верхняя кооридната прорисовки автомобиля
    /// </summary>
    protected int? _startPosY;
    /// <summary>
    /// Ширина прорисовки автомобиля
    /// </summary>
    private readonly int _drawningWidth = 145;
    /// <summary>
    /// Высота прорисовки автомобиля
    /// </summary>
    private readonly int _drawningHeight = 50;

    /// <summary>
    /// Координата X объекта
    /// </summary>
    public int? GetPosX => _startPosX;

    /// <summary>
    /// Координата Y объекта
    /// </summary>
    public int? GetPosY => _startPosY;

    /// <summary>
    /// Ширина объекта
    /// </summary>
    public int GetWidth => _drawningWidth;

    /// <summary>
    /// Высота объекта
    /// </summary>
    public int GetHeight => _drawningHeight;

    private DrawningLocomotive()
    {
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }
    /// <summary>
    /// Инициализация свойств
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    public DrawningLocomotive(int speed, double weight, Color bodyColor) : this()
    {
        EntityLocomotive = new EntityLocomotive(speed, weight, bodyColor);
    }

    public DrawningLocomotive(int drawningWidth, int drawningHeight) : this()
    {
        _drawningWidth = drawningWidth;
        _drawningHeight = drawningHeight;
    }

    public DrawningLocomotive(EntityLocomotive locomotive) : this()
    {
        EntityLocomotive = new EntityLocomotive(locomotive.Speed, locomotive.Weight, locomotive.BodyColor);
    }

    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя  разместить объект в этих размерах</returns>
    public bool SetPictureSize(int width, int height)
    {
        // проверка, что объект "влезает" в размеры поля
        // если влезает, сохраняем границы и корректируем позицию объекта, если она была уже установлена
        if (width >= _drawningWidth && height >= _drawningHeight)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            return true;
        }
        return false;
    }
    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }
        // TODO если при установке объекта в эти координаты, он будет "выходить" за границы формы
        // то надо изменить координаты, чтобы он оставался в этих границах

        if (x < 0) 
        {
            x = 0; 
        }
        else if (x + _drawningWidth > _pictureWidth.Value) 
        {
            x = _pictureWidth.Value - _drawningWidth;
        }


        if (y < 0) 
        {
            y = 0; 
        }
        else if (y + _drawningHeight > _pictureHeight.Value)
        {
            y = _pictureHeight.Value - _drawningHeight;
        }


        _startPosX = x;
        _startPosY = y;
    }
    /// <summary>
    /// Изменение направления перемещения
    /// </summary>
    /// <param name="direction">Направление</param>
    /// <returns>true - перемещене выполнено, false - перемещение невозможно</returns>
    public bool MoveTransport(DirectionType direction)
    {
        if (EntityLocomotive == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return false;
        }
        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityLocomotive.Step > 0)
                {
                    _startPosX -= (int)EntityLocomotive.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityLocomotive.Step > 0)
                {
                    _startPosY -= (int)EntityLocomotive.Step;
                }
                return true;
            // вправо
            case DirectionType.Right:
                if (_startPosX.Value + EntityLocomotive.Step + _drawningWidth < _pictureWidth)
                {
                    _startPosX += (int)EntityLocomotive.Step;
                }

                return true;
            //вниз
            case DirectionType.Down:
                if (_startPosY.Value + EntityLocomotive.Step + _drawningHeight < _pictureHeight)
                {
                    _startPosY += (int)EntityLocomotive.Step;
                }
                return true;
            default:
                return false;
        }
    }
    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public virtual void DrawTransport(Graphics g)
    {
        if (EntityLocomotive == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }
        Pen pen = new(Color.Black);


        // Границы
        Brush brush = new SolidBrush(EntityLocomotive.BodyColor);
        g.DrawRectangle(pen, _startPosX.Value + 5, _startPosY.Value + 20, 120, 20);
        Point[] p1 = {
            new Point(_startPosX.Value + 12, _startPosY.Value),
            new Point(_startPosX.Value + 125, _startPosY.Value),
            new Point(_startPosX.Value + 125, _startPosY.Value + 20),
            new Point(_startPosX.Value + 5, _startPosY.Value + 20)
        };
        g.DrawPolygon(pen, p1);

        // Окрас

        g.FillRectangle(brush, _startPosX.Value + 6, _startPosY.Value + 21, 119, 19);

        Point[] p2 = {
            new Point(_startPosX.Value + 12, _startPosY.Value + 1),
            new Point(_startPosX.Value + 125, _startPosY.Value + 1),
            new Point(_startPosX.Value + 125, _startPosY.Value + 20),
            new Point(_startPosX.Value + 5, _startPosY.Value + 20)
        };
        g.FillPolygon(brush, p2);

        //дверь
        Brush brWhite = new SolidBrush(Color.White);
        g.DrawRectangle(pen, _startPosX.Value + 40, _startPosY.Value + 8, 10, 25);
        g.FillRectangle(brWhite, _startPosX.Value + 41, _startPosY.Value + 9, 9, 24);

        //окна
        Pen pen1 = new(Color.Blue);
        g.DrawRectangle(pen1, _startPosX.Value + 13, _startPosY.Value + 4, 10, 11);
        g.DrawRectangle(pen1, _startPosX.Value + 27, _startPosY.Value + 4, 10, 11);
        g.DrawRectangle(pen1, _startPosX.Value + 110, _startPosY.Value + 4, 10, 11);

        // держатели

        Brush brBlack = new SolidBrush(Color.Black);

        //1

        Point[] point1 = {
            new Point(_startPosX.Value + 7, _startPosY.Value + 40),
            new Point(_startPosX.Value + 45, _startPosY.Value + 40),
            new Point(_startPosX.Value + 38, _startPosY.Value + 46),
            new Point(_startPosX.Value, _startPosY.Value + 46)
        };
        g.DrawPolygon(pen, point1);
        g.FillPolygon(brBlack, point1);

        //2

        Point[] point3 = {
            new Point(_startPosX.Value + 123, _startPosY.Value + 40),
            new Point(_startPosX.Value + 130, _startPosY.Value + 46),
            new Point(_startPosX.Value + 92, _startPosY.Value + 46),
            new Point(_startPosX.Value + 85, _startPosY.Value + 40)
        };
        g.DrawPolygon(pen, point3);
        g.FillPolygon(brBlack, point3);


        //колеса

        //1-ое колеса
        g.DrawEllipse(pen, _startPosX.Value + 13, _startPosY.Value + 40, 11, 11);
        g.FillEllipse(brWhite, _startPosX.Value + 13, _startPosY.Value + 40, 11, 11);

        //2-ое колесо
        g.DrawEllipse(pen, _startPosX.Value + 35, _startPosY.Value + 40, 11, 11);
        g.FillEllipse(brWhite, _startPosX.Value + 35, _startPosY.Value + 40, 11, 11);

        //3-ее колесо
        g.DrawEllipse(pen, _startPosX.Value + 85, _startPosY.Value + 40, 11, 11);
        g.FillEllipse(brWhite, _startPosX.Value + 85, _startPosY.Value + 40, 11, 11);

        //4-ое колесо
        g.DrawEllipse(pen, _startPosX.Value + 107, _startPosY.Value + 40, 11, 11);
        g.FillEllipse(brWhite, _startPosX.Value + 107, _startPosY.Value + 40, 11, 11);
    }
}
